from os.path import dirname, basename
from unittest.mock import patch, MagicMock
from ara_tools.directory_navigator import DirectoryNavigator  

import pytest
import os



@pytest.fixture
def navigator():
    return DirectoryNavigator()




from mock import Mock, patch, mock_open
from pathlib import Path
from ara_tools.artefact_creator import ArtefactCreator
from ara_tools.classifier import Classifier  # Adjust this import path according to your setup

import pytest



def test_valid_classifiers_are_correctly_set():
    # Ensure the static valid_classifiers dictionary in Classifier class is correctly set
    expected_classifiers = {
        "vision": "vision",
        "businessgoal": "businessgoals",
        "capability": "capabilities",
        "keyfeature": "keyfeatures",
        "feature": "features",
        "epic": "epics",
        "userstory": "userstories",
        "task": "tasks",
        "tasklist": "tasks",
        "example": "examples",
        "issue": "issues"
    }

    assert Classifier.valid_classifiers == expected_classifiers

@patch("ara_tools.artefact_creator.os.path.exists")
def test_create_file_with_template_and_classifier(mock_exists):
    mock_exists.return_value = True

    # Mock open function to avoid file IO
    m = mock_open()
    with patch("builtins.open", m):
        fc = ArtefactCreator()
        fc.create_file("dummy_path", "template_path", "vision", "dummy_file")

    m.assert_any_call("template_path/template.vision", "r")
    m.assert_any_call("dummy_path", "w")

@patch("ara_tools.artefact_creator.os.path.exists")
def test_create_file_without_template_or_classifier(mock_exists):
    mock_exists.return_value = False

    # Mock open function to avoid file IO
    m = mock_open()
    with patch("builtins.open", m):
        fc = ArtefactCreator()
        fc.create_file("dummy_path")

    m.assert_any_call("dummy_path", "w")

def test_create_directory_with_valid_path():
    # Mock makedirs to avoid directory creation
    mock_fs = Mock()
    fc = ArtefactCreator(mock_fs)
    fc.create_directory("valid_path")
    
    mock_fs.makedirs.assert_called_with("valid_path", exist_ok=True)

def test_template_exists_with_valid_path():
    mock_fs = Mock()
    mock_fs.path.join.return_value = "full_path"
    mock_fs.path.isfile.return_value = True

    fc = ArtefactCreator(mock_fs)
    result = fc.template_exists("template_path", "template_name")

    assert result

def test_run_with_invalid_classifier_prints_error_message(capfd):
    fc = ArtefactCreator()
    fc.run("filename", "invalid_classifier")

    captured = capfd.readouterr()
    assert "Invalid classifier provided. Please provide a valid classifier." in captured.out

@patch("ara_tools.artefact_creator.input", return_value="n")
@patch("ara_tools.artefact_creator.os.path.exists", return_value=True)
def test_run_with_existing_file_does_not_overwrite(mock_input, mock_exists, capfd):
    fc = ArtefactCreator()
    fc.run("filename", "vision")

    captured = capfd.readouterr()
    assert "No changes were made to the existing file and directory." in captured.out

def test_create_artefact_exploration_success():
    creator = ArtefactCreator()

    # Mock the Path's exists method to always return True
    with patch.object(Path, "exists", return_value=True):
        with patch("builtins.open", mock_open()), patch("shutil.copyfile"):
            creator.create_artefact("./dest", "./source", "sample")

def test_create_artefact_exploration_source_not_found():
    creator = ArtefactCreator()

    with patch("pathlib.Path.exists", return_value=False):
        with pytest.raises(FileNotFoundError):
            creator.create_artefact("./dest", "./source", "sample")

def test_create_artefact_exploration_dest_not_found():
    creator = ArtefactCreator()

    with patch.object(Path, "exists", lambda self: "source" in str(self)):
        with pytest.raises(NotADirectoryError):
            creator.create_artefact("./dest", "./source", "sample")




from mock import Mock, patch
from unittest.mock import call, mock_open
from ara_tools.artefact_renamer import ArtefactRenamer  # Adjust this import path according to your setup
from ara_tools.classifier import Classifier  # Adjust this import path according to your setup

import mock  # Add this line at the top of your file
import pytest
import os
import shutil

@pytest.fixture(autouse=True)
def cleanup():
    """
    A fixture to clean up the 'new_name.data' directory after each test case.
    """
    yield  # This is where the test runs
    new_data_dir = "ara/userstories/new_name.data"
    if os.path.exists(new_data_dir):
        shutil.rmtree(new_data_dir)

@patch("ara_tools.artefact_renamer.os.path.exists")
def test_rename_checks_filename_exists(mock_exists):
    mock_exists.return_value = False
    ar = ArtefactRenamer(os)
    with pytest.raises(FileNotFoundError):
        ar.rename("nonexistent_file", "new_file", "vision")

def test_rename_checks_classifier_valid():
    ar = ArtefactRenamer(os)
    with pytest.raises(ValueError):
        ar.rename("existing_file", "new_file", "invalid_classifier")

def test_rename_checks_new_name_provided():
    ar = ArtefactRenamer(os)
    with pytest.raises(ValueError):
        ar.rename("existing_file", "vision", None)

@patch("builtins.open", new_callable=mock_open, read_data="Vision: Old Title\nOther content.")
@patch("ara_tools.artefact_renamer.os.rename")
@patch("ara_tools.artefact_renamer.os.path.exists", side_effect=[True, True, False, False, True])
def test_rename_filename_with_new_name(mock_exists, mock_rename, mock_open):
    ar = ArtefactRenamer(os)
    ar.rename("existing_file", "new_file", "vision")
    assert mock_rename.call_count == 2
    mock_rename.assert_has_calls([
        call("vision/existing_file.vision", "vision/new_file.vision"),
        call("vision/existing_file.data", "vision/new_file.data")
    ])

@patch("ara_tools.artefact_renamer.os.path.exists", side_effect=[True, True, True, False])
def test_rename_throws_error_if_new_file_or_directory_exists(mock_exists):
    ar = ArtefactRenamer(os)
    with pytest.raises(FileExistsError):
        ar.rename("existing_file", "existing_file", "vision")

@patch("ara_tools.artefact_renamer.os.path.exists", side_effect=[True, False, False])
def test_rename_checks_related_data_folder_exists(mock_exists):
    ar = ArtefactRenamer(os)
    with pytest.raises(FileNotFoundError):
        ar.rename("old_name", "new_name", "userstory")

@patch("builtins.open", new_callable=mock_open, read_data="Userstory: Old Title\nOther content.")
@patch("ara_tools.artefact_renamer.os.rename")
@patch("ara_tools.artefact_renamer.os.path.exists", side_effect=[True, True, False, False, True])
def test_rename_also_renames_related_data_folder(mock_exists, mock_rename, mock_open):
    ar = ArtefactRenamer(os)
    ar.rename("old_name", "new_name", "userstory")
    assert mock_rename.call_count == 2
    mock_rename.assert_has_calls([
        call("userstories/old_name.userstory", "userstories/new_name.userstory"),
        call("userstories/old_name.data", "userstories/new_name.data")
    ])


@patch("ara_tools.artefact_renamer.os.path.exists", side_effect=[True, True, True])
def test_rename_throws_error_if_new_data_folder_exists(mock_exists):
    ar = ArtefactRenamer(os)
    with pytest.raises(FileExistsError):
        ar.rename("old_name", "new_name", "userstory")


@pytest.mark.parametrize("classifier,artefact_name,read_data_prefix,old_title,new_title", [
    ("vision", "Vision", "Vision: ", "Old Title", "New title"),
    ("businessgoal", "Businessgoal", "Businessgoal: ", "Old Title", "New title"),
    ("capability", "Capability", "Capability: ", "Old Title", "New title"),
    ("keyfeature", "Keyfeature", "Keyfeature: ", "Old Title", "New title"),
    ("feature", "Feature", "Feature: ", "Old Title", "New title"),
    ("epic", "Epic", "Epic: ", "Old Title", "New title"),
    ("userstory", "Userstory", "Userstory: ", "Old Title", "New title"),
    ("task", "Task", "Task: ", "Old Title", "New title"),
    ("task", "Task list", "Task list: ", "Old Title", "New title"),
    ("example", "Example", "Example: ", "Old Title", "New title"),
])


@patch("builtins.open", new_callable=mock_open)
def test_update_title_in_artefact(mock_file, classifier, artefact_name, read_data_prefix, old_title, new_title):
    ar = ArtefactRenamer(os)
    read_data = f"{read_data_prefix}{old_title}\nOther content that remains unchanged."
    mock_file.return_value.read = mock.Mock(return_value=read_data)
    artefact_path = f"path/to/{classifier}.artefact"

    # Ensure that the mock for get_artefact_title returns the prefix without an extra colon and space
    with patch.object(Classifier, 'get_artefact_title', return_value=artefact_name):
        ar._update_title_in_artefact(artefact_path, new_title, classifier)

    # Check that the file was opened for reading
    mock_file.assert_any_call(artefact_path, 'r')
    # Check that the file was opened for writing
    mock_file.assert_any_call(artefact_path, 'w')
    # Check that the file write was called with the correct new content
    expected_content = read_data.replace(f"{read_data_prefix}{old_title}", f"{read_data_prefix}{new_title}")
    mock_file().write.assert_called_with(expected_content)






from os import environ, scandir, makedirs, remove, walk, getenv, rmdir
from os.path import join, exists, isdir, getsize, isfile, getctime
from json import load, dump
from concurrent.futures import ThreadPoolExecutor, wait
from shutil import rmtree
from ara_tools.template_manager import DirectoryNavigator
from langchain_community.document_loaders import TextLoader
from langchain_community.vectorstores import Chroma
from langchain.embeddings.openai import OpenAIEmbeddings
from langchain.schema import Document

environ['OPENAI_API_KEY'] = getenv("GPT_KEY")


def add_paths(paths_to_add: list):
    config_file_path="./ara/.vector_db/vector_config.json"
    data = set()

    if exists(config_file_path) and getsize(config_file_path) > 0:
        with open(config_file_path, 'r') as file:
            data = set(load(file))

    for path in paths_to_add:
        add_path_if_exists(data, path)
    
    if data:
        update_db_for_paths(data)
    

def update_db_for_paths(data):
    config_file_path="./ara/.vector_db/vector_config.json"
    with open(config_file_path, 'w') as file:
        dump(list(data), file, indent=4)
    init_DB()


def add_path_if_exists(data, path):
    if not exists(path):
        print(f"{path} does not exist")
        return
    data.add(path)


def embeddings(document):
    parent_directory = "ara"
    hidden_directory = ".vector_db"
    full_path = join(parent_directory, hidden_directory)

    if not exists(full_path):
        makedirs(full_path)

    embeddings = OpenAIEmbeddings()
    chromadb = Chroma.from_documents(document, embeddings, persist_directory=full_path)
    return chromadb



def loadembeddings():
    parent_directory = "ara"
    hidden_directory = ".vector_db"
    full_path = join(parent_directory, hidden_directory)

    embeddings = OpenAIEmbeddings()
    chromadb_2 = Chroma(persist_directory=full_path, embedding_function=embeddings)
    return chromadb_2


def read_json(path):
    with open(path, 'r') as file:
        return load(file)



def process_document(file_path):
    loader = TextLoader(file_path)

    if "./ara/.vector_db/" in file_path:
        return

    documents = loader.load()

    if documents:
        return embeddings(documents)

    return None


def process_entry(entry, current_depth):
    if entry.is_dir(follow_symlinks=False):
        list_files(entry.path, current_depth + 1)
    if entry.is_file():
        process_document(entry.path)
    if not exists(entry.path):
        print(f"{entry.path} does not exist")


def list_files(start_path, current_depth=0):
    if isfile(start_path):
        process_document(start_path)
        return

    with ThreadPoolExecutor() as executor:
        with scandir(start_path) as entries:
            futures = [executor.submit(process_entry, entry, current_depth) for entry in entries]
            wait(futures)


def process_paths(file_paths):
    if not file_paths:
        print("vectorDB successfully initialized.")
        return

    path = file_paths[0]
    if not exists(path):
        print(f"{path} does not exist")
        return
    
    list_files(path)
    process_paths(file_paths[1:])


def create_DB():
    config_file_path = "./ara/.vector_db/vector_config.json"

    if exists(config_file_path):
        init_DB()
        return

    makedirs("./ara/.vector_db")

    with open(config_file_path, "x") as file:
        file.write('[]')
    print("vectorDB config file successfully created.")


def update_DB():
    config_file_path = "./ara/.vector_db/vector_config.json"

    if not exists(config_file_path):
        create_DB()
        return
    
    init_DB()


def init_DB():
    config_file_path = "./ara/.vector_db/vector_config.json"

    with open(config_file_path, 'r') as file:
        file_paths = load(file)
        if file_paths == []:
            print("vector config contains no paths.")
            return
        process_paths(file_paths)


def reset_DB():
    directory = './ara/.vector_db'
    file_to_keep = 'vector_config.json'
    full_path_to_keep = join(directory, file_to_keep)

    if not exists(full_path_to_keep):
        create_DB()

    paths_to_delete = []
    all_paths_in_directory = list(walk(directory, topdown=False))

    for root, dirs, files in all_paths_in_directory:
        paths_to_delete.extend(join(root, f) for f in files if join(root, f) != full_path_to_keep)
        paths_to_delete.extend(join(root, d) for d in dirs)

    file_paths = list(filter(isfile, paths_to_delete))
    dir_paths = list(filter(isdir, paths_to_delete))

    list(map(remove, file_paths))
    list(map(rmtree, dir_paths))

    print("vectorDB successfully reset.")



def reset_config():
    if exists("./ara/.vector_db/vector_config.json"):
        with open("./ara/.vector_db/vector_config.json", 'w') as file:
                file.write('[]')
        print("vectorDB config successfully reset.")
        return


def search_DB(amount, query):
    if not exists("./ara/.vector_db/chroma.sqlite3"):
        init_DB()
        return
    db = loadembeddings()
    docs = db.similarity_search(query, amount)
    metadata_content = ""
    metadata_content += f'Query: "{query}"\n\n'
    count = 0

    metadata_content += "\n".join(
        f'{i + 1}. "{result.metadata.get("source", "No source available")}"'
        for i, result in enumerate(docs)
    )

    with open("./ara/.vector_db/recommendations-vector.md", 'w') as file:
        file.write(metadata_content)


# artefact_deleter.py
import os
import shutil
from ara_tools.template_manager import DirectoryNavigator
from ara_tools.classifier import Classifier
from ara_tools.artefact_link_updater import ArtefactLinkUpdater

class ArtefactDeleter:
    def __init__(self, file_system=None):
        self.file_system = file_system or os
        self.link_updater = ArtefactLinkUpdater()

    def delete(self, filename, classifier):
        navigator = DirectoryNavigator()
        navigator.navigate_to_target()

        if not Classifier.is_valid_classifier(classifier):
            print("Invalid classifier provided. Please provide a valid classifier.")
            return

        sub_directory = Classifier.get_sub_directory(classifier)
        file_path = self.file_system.path.join(sub_directory, f"{filename}.{classifier}")
        dir_path = self.file_system.path.join(sub_directory, f"{filename}.data")

        if not self.file_system.path.exists(file_path) or not self.file_system.path.exists(dir_path):
            print("File or directory not found.")
            return

        user_choice = input("Are you sure you want to delete the file and directory? (Y/N): ")

        if user_choice.lower() != "y":
            print("No changes were made.")
            return

        # Remove references to this artefact in other artefacts before deletion
        self.link_updater.remove_links_in_related_artefacts(filename)

        self.file_system.remove(file_path)
        shutil.rmtree(dir_path)

        print(f"Deleted file: {file_path}")
        print(f"Deleted directory: {dir_path}")



import os
from ara_tools.prompt_handler import send_prompt

class Chat:
    def __init__(self, chat_name):
        self.chat_name = self.setup_chat(chat_name)
        self.chat_history = []

    def setup_chat(self, chat_name):
        if chat_name.endswith('.md'):
            return self.handle_existing_chat(chat_name)
        if chat_name == "chat" and os.path.exists("chat.md"):
            return self.handle_existing_chat("chat.md")
        return self.initialize_new_chat(chat_name)

    def handle_existing_chat(self, chat_name):
        if not os.path.exists(chat_name):
            print("Couldn't continue existing chat. Provided chat file doesn't exist in this directory.")
            sys.exit(1)
        self.chat_history = self.load_chat_history(chat_name)
        print(f"Reloaded {chat_name} content:")
        return chat_name

    def initialize_new_chat(self, chat_name):
        if chat_name != "chat":
            chat_name = f"{chat_name}-chat"
        chat_name = f"{chat_name}.md"
        open(chat_name, 'a', encoding='utf-8').close()
        return chat_name

    def start(self):
        print("Start chatting (type 'GO' to send, 'END' to finish):")
        while True: 
            user_input = self.get_user_input()
            if user_input == "END":
                print("Chat ended")
                return
            self.handle_user_input(user_input)

    def get_user_input(self):
        user_input = ""
        while True:
            line = input()
            if line in ["END", "GO", "CONTINUE"]:
                return f"{user_input.strip()}" if line == "GO" else line
            user_input += f"{line} "

    def handle_user_input(self, user_input):
        if user_input == "CONTINUE":
            # Delete last response if it exists and resend chat history up to the last prompt
            self.resend_message()
        else:
            # Don't save message if command was continue
            self.save_message("You", user_input)
        prompt_to_send = "\n".join([message for message in self.chat_history])
        response = send_prompt(prompt_to_send)
        print(f"GPT-4: {response}")
        self.save_message("ChatGPT", response)

    def resend_message(self):
        with open(self.chat_name, 'r', encoding='utf-8') as file:
            lines = file.readlines()
        if lines:
            index_to_remove = self.find_last_reply_index(lines)
            if index_to_remove is not None:
                with open(self.chat_name, 'w', encoding='utf-8') as file:
                    file.writelines(lines[:index_to_remove])
            self.chat_history = self.load_chat_history(self.chat_name)

    def find_last_reply_index(self, lines):
        index_to_remove = None
        for i, line in enumerate(reversed(lines)):
            if "#### You" in line:
                # Last reply was already deleted
                break
            if "#### ChatGPT" in line:
                # Delete last reply
                index_to_remove = len(lines) - i - 1
                break
        return index_to_remove

    def save_message(self, role, message):
        line_to_write = f"#### {role}:\n{message}\n\n"
        with open(self.chat_name, 'a', encoding='utf-8') as file:
            file.write(line_to_write)
        self.chat_history.append(line_to_write)

    def load_chat_history(self, chat_file):
        chat_history = []
        if os.path.exists(chat_file):
            with open(chat_file, 'r', encoding='utf-8') as file:
                chat_history = file.readlines()
        return chat_history


from unittest.mock import patch, MagicMock
from pathlib import Path
from ara_tools.classifier import Classifier
from ara_tools.template_manager import SpecificationBreakdownAspects, ArtefactFileManager
from ara_tools.directory_navigator import DirectoryNavigator
from ara_tools.template_manager import TemplatePathManager
from io import StringIO

import pytest
import os



@pytest.fixture(autouse=True)
def navigate_to_ara_directory():
    navigator = DirectoryNavigator("ara")
    original_directory = navigator.navigate_to_target()
    yield
    os.chdir(original_directory)


@pytest.mark.parametrize(
    "file_exists, dir_exists, expected_mkdir_calls, expected_chdir_calls",
    [
        (True, True, 0, 1),
        (True, False, 1, 1),
        (False, False, 0, 0)
    ]
)
def test_create_directory(file_exists, dir_exists, expected_mkdir_calls, expected_chdir_calls):
    artefact_file = 'test_artefact'
    data_dir = 'test_artefact.data'
    sba = ArtefactFileManager()

    with patch('os.path.isfile', return_value=file_exists), \
         patch('os.path.exists', return_value=dir_exists), \
         patch('os.mkdir') as mock_mkdir, \
         patch('os.chdir') as mock_chdir:

        if not file_exists:
            with pytest.raises(ValueError, match=f"File {artefact_file} does not exist. Please create it first."):
                sba.create_directory(artefact_file, data_dir)
        else:
            sba.create_directory(artefact_file, data_dir)

        assert mock_mkdir.call_count == expected_mkdir_calls
        assert mock_chdir.call_count == expected_chdir_calls


@pytest.mark.parametrize(
    "aspect, expect_exception, match_str",
    [
        ("technology", False, None),
        ("", True, f"Template file .* does not exist."),
        ("invalid", True, f"Template file .* does not exist.")
    ]
)
def test_copy_templates_to_directory(aspect, expect_exception, match_str):
    sba = ArtefactFileManager()

    if expect_exception:
        with pytest.raises(FileNotFoundError, match=match_str):
            sba.copy_templates_to_directory(aspect)
    else:
        with patch("ara_tools.template_manager.copy") as mock_copy:
            sba.copy_templates_to_directory(aspect)
            mock_copy.assert_called()


@pytest.mark.parametrize(
    "file_exists, dir_exists, raises, expected_exception, expected_message",
    [
        (True, True, False, None, None),
        (True, False, False, None, None),
        (False, False, True, ValueError, f"File .* does not exist. Please create it first.")
    ]
)
def test_create(file_exists, dir_exists, raises, expected_exception, expected_message):
    artefact_name = 'test_artefact'
    classifier = 'capability'
    aspect = 'technology'
    sba = SpecificationBreakdownAspects()

    with patch('os.path.isfile', return_value=file_exists), \
         patch('os.path.exists', return_value=dir_exists), \
         patch('os.mkdir'), patch('os.chdir'), patch('shutil.copy'):

        if raises:
            with pytest.raises(expected_exception, match=expected_message):
                sba.create(artefact_name, classifier, aspect)
        else:
            sba.create(artefact_name, classifier, aspect)


def test_get_template_content_with_path_mock():
    classifier = "test"
    expected_content = "Hello, this is a test template."
    mock_base_path = MagicMock()

    mock_template_path = MagicMock()
    mock_base_path.__truediv__.return_value = mock_template_path

    mock_file = MagicMock()
    mock_file.read.return_value = expected_content
    mock_template_path.open.return_value.__enter__.return_value = mock_file

    with patch.object(
        TemplatePathManager, 'get_template_base_path_artefacts', return_value=mock_base_path
    ), patch("sys.stdout", new_callable=StringIO) as captured_output:
        manager = TemplatePathManager()

        # We're capturing stdout in as captured_output
        manager.get_template_content(classifier)

        assert captured_output.getvalue().strip() == expected_content


# Add more tests as needed



from langchain_openai import ChatOpenAI
from ara_tools.classifier import Classifier
from os.path import exists, join, splitext, isfile
from os import makedirs, listdir, environ
from sys import exit
from subprocess import run
from re import findall

class ChatOpenAISingleton:
    _instance = None

    def __init__(self):
        ChatOpenAISingleton._instance = ChatOpenAI(openai_api_key=environ.get("GPT_KEY"))

    @staticmethod
    def get_instance():
        if ChatOpenAISingleton._instance is None:
            ChatOpenAISingleton()
        return ChatOpenAISingleton._instance    

def write_string_to_file(filename, string, mode):
    with open(filename, mode) as file:
            file.write(f"\n{string}\n")
    return file

def read_string_from_file(path):
    with open(path, 'r') as file:
            text = file.read()
    return text

def read_prompt(classifier, param):
    sub_directory = Classifier.get_sub_directory(classifier)
    prompt_path = f"ara/{sub_directory}/{param}.data/prompt.data/{classifier}.prompt"

    prompt = read_string_from_file(prompt_path)
    return prompt

def send_prompt(prompt):
    chat = ChatOpenAISingleton.get_instance()
    chat_result = chat.invoke(prompt)
    return chat_result.content

def append_headings(classifier, param, heading_name):
    sub_directory = Classifier.get_sub_directory(classifier)

    artefact_data_path = f"ara/{sub_directory}/{param}.data/{classifier}_exploration.md"
    content = read_string_from_file(artefact_data_path)
    pattern = r'## {}_(\d+)'.format(heading_name)
    matches = findall(pattern, content)

    max_number = 1
    if matches:
        max_number = max(map(int, matches)) + 1
    heading = f"## {heading_name}_{max_number}"
            
    write_string_to_file(artefact_data_path, heading, 'a')

def write_prompt_result(classifier, param, text):
    sub_directory = Classifier.get_sub_directory(classifier)

    artefact_data_path = f"ara/{sub_directory}/{param}.data/{classifier}_exploration.md"
    write_string_to_file(artefact_data_path, text, 'a')


def prompt_creation(classifier, parameter):
    sub_directory = Classifier.get_sub_directory(classifier)
    
    artefact_path = join(f"ara/{sub_directory}/{parameter}.data")
    if not exists(artefact_path):
        print(f"ERROR: {classifier} {parameter} in does not exist.")
        exit(1)

    prompt_data_path = f"ara/{sub_directory}/{parameter}.data/prompt.data"
    makedirs(prompt_data_path, exist_ok=True)
    ensure_basic_prompt_files_exists(classifier, prompt_data_path)
    create_prompt(classifier, prompt_data_path)

def ensure_basic_prompt_files_exists(classifier, prompt_data_path):
    """
    Ensures that all basic prompt files for a given classifier exist within the specified directory and creates them if they don't.
    
    :param classifier: The type of artefact (e.g., 'feature', 'vision'), which determines the template used for file creation.
    :param prompt_data_path: The path to the directory where 'prompt.data' files are stored.
    :return: Returns 'False' if any file was created during the execution, indicating that the user should review these files.
             Returns 'True' if all files were already in place.

    Note: 
    - The function prints a message listing the newly created files, if any.
    - The content for 'prompt.md' and 'givens.md' requires further manual editing.
    """
    basic_prompt_files = ["commands.md", "givens.md", "prompt.md", "rules.md"]
    basic_prompt_files = set(filter(lambda file: need_to_be_created(file, prompt_data_path), basic_prompt_files))
    
    for basic_file in basic_prompt_files:
        file_path = join(prompt_data_path, basic_file)
        with open(file_path, 'w') as file:
            content = basic_prompt_file_content_creation(classifier, basic_file)
            file.write(content)

    if basic_prompt_files:
        print(f"Following File(s) were created: {basic_prompt_files}. \nCheck these files and continue by re-entering the command.")
        exit(1)

def need_to_be_created(basic_prompt_file, prompt_data_path):
    path = join(prompt_data_path, basic_prompt_file)
    return not exists(path)

def basic_prompt_file_content_creation(classifier, basic_prompt_file):
    content = ""
    if basic_prompt_file in ("commands.md", "rules.md", "prompt.md"):
        content = get_template_content(classifier, basic_prompt_file)

    if basic_prompt_file == "givens.md":                    
        command = ["ara", "list"]
        output = run(command, capture_output=True, text=True).stdout
        output = output.replace(" - ", " - [ ] ")
        content = output

    return content

def get_template_content(classifier, basic_prompt_file):
    basic_prompt_template_path = f"ara_tools/templates/prompt-creation"
    root, _ = splitext(basic_prompt_file)
    template_name = f"template_{classifier}.md"
    
    root = root.replace("prompt", "prompts")

    template_path = join(basic_prompt_template_path, root, template_name)
    content = ""
    
    if not exists(template_path):
        print(f"WARNING: {template_path} does not exist. Please create a template or {basic_prompt_file} will be empty!")
        return f"# {basic_prompt_file} \n\nNo template found, fill manually or create a template ({template_path})!"

    with open(template_path, 'r') as file:
        return file.read()
    

def create_prompt(classifier, prompt_data_path):
    prompt_file_path = join(prompt_data_path, f"{classifier}.prompt")
    combined_content = ""
    
    md_files = listdir(prompt_data_path)
    md_files = set(filter(lambda file: is_prompt_file(file, prompt_data_path), md_files))
    md_files.remove('givens.md')

    for file_name in md_files:
        md_prompt_file_path = join(prompt_data_path, file_name)
        combined_content += get_file_content(md_prompt_file_path) + "\n\n"

    given_content = "# givens \n\n"
    md_prompt_file_path = join(prompt_data_path, "givens.md")
    for item in extract_checked_items(md_prompt_file_path):
        given_item_path = join("ara", item)
        given_content += get_file_content(given_item_path) + "\n\n"

    combined_content += given_content
    with open(prompt_file_path, 'w') as file:
        file.write(combined_content)


def is_prompt_file(file, prompt_data_path):
    path = join(prompt_data_path, file)
    if exists(path) and file.endswith(".md"):
        return True
    return False


def get_file_content(path):
    with open(path, 'r') as file:
        return file.read()


def get_checked_lines(line):
    return line.startswith("  - [x] ")


def get_path_only(line):
    return line.replace("  - [x] ./", "").strip()


def extract_checked_items(file_path):
    with open(file_path, 'r') as file:

        lines = filter(get_checked_lines, file)
        lines = map(get_path_only, lines)
        lines = set(lines)

    print(lines)
    return lines


# artefact_lister.py
from ara_tools.file_classifier import FileClassifier
from ara_tools.template_manager import DirectoryNavigator
import os

class ArtefactLister:
    def __init__(self, file_system=None):
        self.file_system = file_system or os

    def list_files(self, tags=None):
        # make sure this function is always called from the ara top level directory
        navigator = DirectoryNavigator()
        navigator.navigate_to_target()
        
        file_classifier = FileClassifier(self.file_system)
        classified_files = file_classifier.classify_files(tags=tags)
        file_classifier.print_classified_files(classified_files)



from ara_tools.classifier import Classifier

class FileClassifier:
    def __init__(self, file_system):
        self.file_system = file_system
    
    def classify_files(self, tags=None):
        files_by_classifier = {classifier: [] for classifier in Classifier.ordered_classifiers()}

        for root, _, files in self.file_system.walk("."):
            for file in files:
                file_path = self.file_system.path.join(root, file)
                if tags:
                    with open(file_path, 'r') as f:
                        content = f.read()
                        # Ensure all tags are in content
                        if not all(tag in content for tag in tags):
                           # print(f"DEBUG: Skipping file {file_path} due to missing tags")
                            continue
                
                for classifier in Classifier.ordered_classifiers():
                    if file.endswith(f".{classifier}"):
                        files_by_classifier[classifier].append(file_path)
        return files_by_classifier

    def print_classified_files(self, files_by_classifier):
        for classifier, files in files_by_classifier.items():
            if files:
                print(f"{classifier.capitalize()} files:")
                for file in files:
                    print(f"  - {file}")
                print()



# artefact_link_updater.py
import os
import re
from ara_tools.classifier import Classifier

class ArtefactLinkUpdater:

    def __init__(self, file_system=None):
        self.file_system = file_system or os

    def update_links_in_related_artefacts(self, old_name, new_name, dir_path='.'):
        new_name_formatted = new_name.replace('_', ' ').capitalize()
        old_name_pattern = re.compile(f"\\b{old_name.replace(' ', '[ _]').replace('_', '[ _]')}\\b", re.IGNORECASE)

        patterns = {
            re.compile(f"^Contributes to?[ ]+([A-Za-z ]+)?{old_name_pattern.pattern}", re.IGNORECASE | re.MULTILINE): f"Contributes to \\1{new_name_formatted}",
            re.compile(f"^Illustrates?[ ]+([A-Za-z ]+)?{old_name_pattern.pattern}", re.IGNORECASE | re.MULTILINE): f"Illustrates \\1{new_name_formatted}"
        }

        # Iterate over all items in the directory
        for item in self.file_system.listdir(dir_path):
            item_path = self.file_system.path.join(dir_path, item)
            extension = os.path.splitext(item)[-1][1:]
            
            # Check if it's a directory, then recurse
            if self.file_system.path.isdir(item_path):
                self.update_links_in_related_artefacts(old_name, new_name, item_path)
            
            # Check if it's a file and not a directory
            # Check if extension is a valid artefact classifier
            elif self.file_system.path.isfile(item_path) and Classifier.is_valid_classifier(extension):
                # Read the content of the file
                with open(item_path, 'r') as file:
                    content = file.read()
                
                # Replace all occurrences of the old name with the new name using regular expressions
                for pattern, new in patterns.items():
                    content = pattern.sub(new, content)
                
                # Write the updated content back to the file
                with open(item_path, 'w') as file:
                    file.write(content)

    def remove_links_in_related_artefacts(self, artefact_name, dir_path="."):
        artefact_name_pattern = re.compile(rf"\b{re.escape(artefact_name)}\b", re.IGNORECASE)

        # Pattern for removing the artefact name from 'Contributes to' lines
        contribute_pattern = re.compile(rf"^(Contributes to).*{artefact_name_pattern.pattern}.*$", re.IGNORECASE | re.MULTILINE)
        
        # Pattern for removing the artefact name from 'Illustrates' lines (no colon)
        illustrates_pattern = re.compile(rf"^(Illustrates).*{artefact_name_pattern.pattern}.*$", re.IGNORECASE | re.MULTILINE)

        # Iterate over all items in the directory
        for item in self.file_system.listdir(dir_path):
            item_path = self.file_system.path.join(dir_path, item)
            extension = os.path.splitext(item)[-1][1:]
            
            # Check if it's a directory, then recurse
            if self.file_system.path.isdir(item_path):
                self.remove_links_in_related_artefacts(artefact_name, item_path)
            
            # Check if it's a file and not a directory, and if extension is a valid artefact classifier
            elif self.file_system.path.isfile(item_path) and Classifier.is_valid_classifier(extension):
                with open(item_path, 'r') as file:
                    content = file.read()
                
                # Remove the artefact name from 'Contributes to' and 'Illustrates' lines
                content = contribute_pattern.sub("Contributes to", content)
                content = illustrates_pattern.sub("Illustrates", content)
                
                with open(item_path, 'w') as file:
                    file.write(content)




from unittest.mock import patch, Mock, mock_open, MagicMock, call
from ara_tools.artefact_renamer import ArtefactRenamer  # Adjust this import path according to your setup
from ara_tools.artefact_link_updater import ArtefactLinkUpdater  # Adjust this import path according to your setup

import pytest
import mock  # Add this line at the top of your file
import os



# Test case for updating link consistency within related artefact files


@patch("ara_tools.artefact_link_updater.os.listdir")
@patch("ara_tools.artefact_link_updater.os.path.isfile")
def test_update_links_in_related_artefacts(mock_isfile, mock_listdir):
    # Setup
    old_name = "Old_name"
    new_name = "New name"
    dir_path = "path/to/artefacts"
    file_contents = {
        "contributing_artefact.userstory": "Contributes to Old_name\nSome other content",
        "illustrating.example": "Illustrates Old_name\nSome other content"
    }

    # Mock the listdir to return a list of files
    mock_listdir.return_value = list(file_contents.keys())

    # Mock the isfile to return True, assuming all listdir entries are files
    mock_isfile.return_value = True

    # Mock the open function using mock_open
    mock_file_handles = {filename: mock_open(read_data=content).return_value
                         for filename, content in file_contents.items()}
    mock_open_function = mock_open()
    mock_open_function.side_effect = lambda file_path, mode='r': mock_file_handles[os.path.basename(file_path)]

    with patch("builtins.open", mock_open_function):
        # Instantiate the ArtefactLinkUpdater with the mocked file system
        link_updater = ArtefactLinkUpdater(file_system=os)

        # Execute the method under test
        link_updater.update_links_in_related_artefacts(old_name, new_name, dir_path)

        # Assertions
        # Verify that the open function was called correctly for each file
        for filename in file_contents.keys():
            expected_file_path = os.path.join(dir_path, filename)
            mock_open_function.assert_any_call(expected_file_path, 'r')
            mock_open_function.assert_any_call(expected_file_path, 'w')

        # Verify the content of the files was updated correctly
        for filename, content in file_contents.items():
            expected_new_content = content.replace(old_name, new_name)
            file_object = mock_file_handles[filename]
            file_object.write.assert_called_with(expected_new_content)


from os.path import join, dirname
import sys
import argparse
from ara_tools.artefact_creator import ArtefactCreator
from ara_tools.artefact_renamer import ArtefactRenamer
from ara_tools.classifier import Classifier
from ara_tools.filename_validator import is_valid_filename
from ara_tools.classifier_validator import is_valid_classifier
from ara_tools.template_manager import SpecificationBreakdownAspects, TemplatePathManager
from ara_tools.artefact_deleter import ArtefactDeleter
from ara_tools.artefact_lister import ArtefactLister
from ara_tools.vectorDB import create_DB, add_paths, reset_config, reset_DB, search_DB, update_DB
from ara_tools.prompt_handler import read_prompt, send_prompt, append_headings, write_prompt_result, prompt_creation
from ara_tools.chat import Chat

def check_validity(condition, error_message):
    if not condition:
        print(error_message)
        sys.exit(1)

def create_action(args):
    if args.parameter and args.classifier and args.aspect:
        sba = SpecificationBreakdownAspects()
        try:
            sba.create(args.parameter, args.classifier, args.aspect)
        except ValueError as ve:
            print(f"Error: {ve}")
            sys.exit(1)

    check_validity(is_valid_filename(args.parameter), "Invalid filename provided. Please provide a valid filename.")
    check_validity(is_valid_classifier(args.classifier), "Invalid classifier provided. Please provide a valid classifier.")

    template_path = join(dirname(__file__), 'templates')
    artefact_creator = ArtefactCreator()
    artefact_creator.run(args.parameter, args.classifier, template_path)

def delete_action(args):
    artefact_deleter = ArtefactDeleter()
    artefact_deleter.delete(args.parameter, args.classifier)


def rename_action(args):
    check_validity(is_valid_filename(args.parameter), "Invalid filename provided. Please provide a valid filename.")
    check_validity(is_valid_classifier(args.classifier), "Invalid classifier provided. Please provide a valid classifier.")
    check_validity(is_valid_filename(args.aspect), "Invalid new filename provided. Please provide a valid filename.")

    artefact_renamer = ArtefactRenamer()
    artefact_renamer.rename(args.parameter, args.aspect, args.classifier)

def list_action(args):
    artefact_lister = ArtefactLister()
    if args.tags:
        artefact_lister.list_files(tags=args.tags)
    else:
        artefact_lister.list_files()
    

def prompt_action(args):
    check_validity(is_valid_classifier(args.classifier), "Invalid classifier provided. Please provide a valid classifier.")
    check_validity(is_valid_filename(args.parameter), "Invalid filename provided. Please provide a valid filename.")

    classifier = args.classifier
    param = args.parameter
    prompt_creation(classifier, param)
    prompt = read_prompt(classifier, param)

    if(prompt):
        append_headings(classifier, param, "prompt")
        write_prompt_result(classifier, param, prompt)
        response = send_prompt(prompt)
        append_headings(classifier, param, "result")
        write_prompt_result(classifier, param, response)

def chat_action(args):
    if args.name:
        # Start or continue specific chat
        chat_name = args.name
    else:
        # Start or continue default chat
        chat_name = "chat"
    chat = Chat(chat_name)
    chat.start()

def vector_action(args): 
    match args.vector_command:
        case 'init':
            create_DB()

        case 'update':
            if args.path:
                add_paths(args.path)
            if not args.path:
                update_DB()

        case 'reset':
            if args.config:
                reset_config()
            if not args.config:
                reset_DB()

        case 'search':
            search_DB(args.amount, args.user_input)

def template_action(args):
    check_validity(is_valid_classifier(args.classifier), "Invalid classifier provided. Please provide a valid classifier.")
    check_validity(Classifier.is_valid_classifier(args.classifier), "Invalid classifier provided. Please provide a valid classifier.")

    template_manager = TemplatePathManager()
    content = template_manager.get_template_content(args.classifier)

def handle_invalid_action(args):
    sys.exit("Invalid action provided. Type ara -h for help")

def create_parser(subparsers):
    create_parser = subparsers.add_parser("create", help="Create a file with a classifier and aspect")
    create_parser.add_argument("parameter", help="Filename for create action")
    create_parser.add_argument("classifier", help="Classifier for the file to be created")
    create_parser.add_argument("aspect", help="Specification breakdown aspect", nargs='?', default=None)

def delete_parser(subparsers):
    delete_parser = subparsers.add_parser("delete", help="Delete a file with a classifier")
    delete_parser.add_argument("parameter", help="Filename for delete action")
    delete_parser.add_argument("classifier", help="Classifier for the file to be deleted")

def rename_parser(subparsers):
    rename_parser = subparsers.add_parser("rename", help="Rename a file with a classifier and new filename")
    rename_parser.add_argument("parameter", help="Current filename for rename action")
    rename_parser.add_argument("classifier", help="Classifier for the file to be renamed")
    rename_parser.add_argument("aspect", help="New filename for the file")

def list_parser(subparsers):
    list_parser = subparsers.add_parser("list", help="List files with optional tags")
    list_parser.add_argument("tags", nargs="*", help="Tags for listing files")

def prompt_parser(subparsers):
    prompt_parser = subparsers.add_parser("prompt", help="Creates a prompt and sends it to ChatGPT and save the result")
    prompt_parser.add_argument("parameter", help="Filename for prompt action")
    prompt_parser.add_argument("classifier", help="Classifier for the file to be prompted")

def vector_init_parser(vector_subparsers):
    init_parser = vector_subparsers.add_parser("init", help="Initialize vectorDB")

def vector_update_parser(vector_subparsers):
    update_parser = vector_subparsers.add_parser("update", help="Update vectorDB")
    update_parser.add_argument("path", nargs='*', help="Path(s) to add to config file", default=None)

def vector_reset_parser(vector_subparsers):
    reset_parser = vector_subparsers.add_parser("reset", help="Reset vectorDB")
    reset_parser.add_argument("config", nargs="?", help="Optional config to reset", default=None)

def vector_search_parser(vector_subparsers):
    search_parser = vector_subparsers.add_parser("search", help="Search vectorDB")
    search_parser.add_argument("amount", nargs="?", help="Amount of paths to return from search", default=4, type=int)
    search_parser.add_argument("user_input", help="Required query to be executed on vectorDB")

def vector_parser(subparsers):
    vector_parser = subparsers.add_parser("vector")
    vector_subparsers = vector_parser.add_subparsers(dest='vector_command')

    vector_init_parser(vector_subparsers)
    vector_update_parser(vector_subparsers)
    vector_reset_parser(vector_subparsers)
    vector_search_parser(vector_subparsers)

def chat_parser(subparsers):
    chat_parser = subparsers.add_parser("chat", help="Command line chatbot")
    chat_parser.add_argument("name", help="Optional name for a specific chat. Pass the .md file to continue an existing chat", nargs='?', default=None)

def template_parser(subparsers):
    template_parser = subparsers.add_parser("template", help="Output ara template")
    template_parser.add_argument("classifier", help="Classifier for ara artefact type")

def cli():
    parser = argparse.ArgumentParser(description="Ara tools for creating files and directories.")
    subparsers = parser.add_subparsers(dest="action", help="Action to perform")

    create_parser(subparsers)
    delete_parser(subparsers)
    rename_parser(subparsers)
    list_parser(subparsers)
    prompt_parser(subparsers)
    vector_parser(subparsers)
    chat_parser(subparsers)
    template_parser(subparsers)

    action_mapping = {
        "create": create_action,
        "delete": delete_action,
        "rename": rename_action,
        "list": list_action,
        "prompt": prompt_action,
        "vector": vector_action,
        "chat": chat_action,
        "template": template_action
    }

    args = parser.parse_args()
    action = action_mapping.get(args.action, handle_invalid_action)
    action(args)


if __name__ == "__main__":
    cli()



import os
from ara_tools.classifier import Classifier
from ara_tools.artefact_link_updater import ArtefactLinkUpdater
from ara_tools.template_manager import DirectoryNavigator
from pathlib import Path

class ArtefactRenamer:
    def __init__(self, file_system=None):
        self.file_system = file_system or os
        self.link_updater = ArtefactLinkUpdater()

    def rename(self, old_name, new_name, classifier):
        # Navigate to the target directory using DirectoryNavigator
        navigator = DirectoryNavigator()
        navigator.navigate_to_target()

        if not new_name:
            raise ValueError("New name must be provided for renaming.")

        if not Classifier.is_valid_classifier(classifier):
            raise ValueError("Invalid classifier provided. Please provide a valid classifier.")

        sub_directory = Classifier.get_sub_directory(classifier)
        old_file_path = self.file_system.path.join(sub_directory, f"{old_name}.{classifier}")
        new_file_path = self.file_system.path.join(sub_directory, f"{new_name}.{classifier}")

        old_dir_path = self.file_system.path.join(sub_directory, f"{old_name}.data")
        new_dir_path = self.file_system.path.join(sub_directory, f"{new_name}.data")

        old_dir_exists = self.file_system.path.exists(old_dir_path)

        # Check if the original file exists
        if not self.file_system.path.exists(old_file_path):
            raise FileNotFoundError(f"The file {old_file_path} does not exist.")

        # Check if the new file name and directory already exist
        if self.file_system.path.exists(new_file_path):
            raise FileExistsError(f"The new file name {new_file_path} already exists.")
        if self.file_system.path.exists(new_dir_path):
            raise FileExistsError(f"The new directory name {new_dir_path} already exists.")

        # Perform the renaming of the file and directory
        self.file_system.rename(old_file_path, new_file_path)
        if old_dir_exists:
            self.file_system.rename(old_dir_path, new_dir_path)
        # TODO: we should be able to remove this next line once ara delete ignores non-existent data folders
        os.makedirs(new_dir_path, exist_ok=True)

        print(f"Renamed file: {old_file_path} to {new_file_path}.")
        print(f"Renamed directory: {old_dir_path} to {new_dir_path}.")


        # Update the title within the artefact file
        self._update_title_in_artefact(new_file_path, new_name, classifier)

        # Update links in related artefact files
        self.link_updater.update_links_in_related_artefacts(old_name, new_name)


  
    def _update_title_in_artefact(self, artefact_path, new_title, classifier):
    # Format the new title: replace underscores with spaces and capitalize the first letter
        formatted_new_title = new_title.replace('_', ' ').capitalize()

        # Get the artefact title prefix using the classifier
        title_prefix = Classifier.get_artefact_title(classifier.lower())
        if not title_prefix:
            raise ValueError(f"Invalid classifier: {classifier}")

        # Read the file content
        with open(artefact_path, 'r') as file:
            content = file.read()

        # Find the old title line
        old_title_line = next((line for line in content.split('\n') if line.startswith(title_prefix)), None)
        if old_title_line is None:
            raise ValueError(f"The artefact file does not contain the title prefix '{title_prefix}'.")

        # Construct the new title line without adding an extra colon
        new_title_line = f"{title_prefix}: {formatted_new_title}"
        # Replace the old title line with the new title line in the content
        new_content = content.replace(old_title_line, new_title_line, 1)

        # Write the updated content back to the file
        with open(artefact_path, 'w') as file:
            file.write(new_content)